unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Buttons, ExtCtrls, Menus, iClip, ClipBrd;

type
	TClipMode = (cmNone, cmRecording, cmPlaying);

type
  TMainForm = class(TForm)
    LB: TListBox;
    Panel1: TPanel;
    SP1: TSpeedButton;
    SP2: TSpeedButton;
    SP3: TSpeedButton;
    SP4: TSpeedButton;
    SP5: TSpeedButton;
    SP6: TSpeedButton;
    pnlState: TPanel;
    Menu: TMainMenu;
    mnuFile: TMenuItem;
    mnuClip: TMenuItem;
    mnuExit: TMenuItem;
    mnuFirst: TMenuItem;
    mnuLast: TMenuItem;
    PLAY1: TMenuItem;
    mnuPlay: TMenuItem;
    mnuRecord: TMenuItem;
    mnuStop: TMenuItem;
    mnuMisc: TMenuItem;
    mnuConf: TMenuItem;
    mnuInfo: TMenuItem;
    mnuClear: TMenuItem;
    mnuSave: TMenuItem;
    Save: TSaveDialog;
    Open: TOpenDialog;
    mnuOpen: TMenuItem;
    N1: TMenuItem;
    mnuCRLF: TMenuItem;
    mnuFLDR: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure LBData(Control: TWinControl; Index: Integer;
      var Data: String);
    procedure LBDrawItem(Control: TWinControl; Index: Integer; Rect: TRect;
      State: TOwnerDrawState);
    procedure mnuRecordClick(Sender: TObject);
    procedure mnuPlayClick(Sender: TObject);
    procedure mnuStopClick(Sender: TObject);
    procedure mnuFirstClick(Sender: TObject);
    procedure mnuLastClick(Sender: TObject);
    procedure mnuExitClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure mnuConfClick(Sender: TObject);
    procedure mnuInfoClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure mnuClearClick(Sender: TObject);
    procedure LBKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure mnuSaveClick(Sender: TObject);
    procedure mnuOpenClick(Sender: TObject);
    procedure mnuCRLFClick(Sender: TObject);
    procedure mnuFLDRClick(Sender: TObject);
  private
    ClipMode    : TClipMode;
    ClipWatcher : TClipWatcher;
    PasteWatcher: TPasteWatcher;

    ClipList    : TStringList;
    Bitmap      : TBitmap;

    DefaultItem : string;

    WithCrLf    : Boolean;

    procedure   AddClipList(AText: string);

    procedure   GetClip();
    procedure	Render();
  public
    procedure   InitRecording();
    procedure   InitPlaying();
    procedure   InitStopping();
  end;

var
  MainForm: TMainForm;

implementation

uses
    iForm, Conf, Info, IniFiles, ShellApi;

{$R *.dfm}

const
    DATA_FILE = 'youcantrashme.dat';

var
    DataFilePath: string;

(*
	=======================================================
    ShellExecutẽbp[֐
	=======================================================
*)

function    ExecuteFile(FilePath: string): Boolean;
const
    ERROR_NO = 32;
var
    Value: integer;
begin
    Value := ShellExecute(
                    Application.MainForm.handle,
                    'open',
                    PChar(FilePath),
                    '',
                    PChar(ExtractFilePath(Application.Exename)),
                    SW_SHOWDEFAULT
                    );
    Result := (Value > ERROR_NO);
end;

(*
	=======================================================
    Rs[o
	=======================================================
*)
procedure   TMainForm.AddClipList(AText: string);
begin
    ClipList.Add(AText);
    LB.Count := ClipList.Count;
    LB.ItemIndex := 0;
end;

procedure   TMainForm.GetClip();
begin
    if Clipboard.HasFormat(CF_TEXT)
    and (ClipMode = cmRecording) then
    begin
        AddClipList(ClipBoard.AsText);
    end;
end;

(*
	=======================================================
    y[Xgo
	=======================================================
*)
procedure	TMainForm.Render();
var
    Txt: String;
    Data: THandle;
    Buff: Pointer;
begin
	inherited;

    if ClipMode = cmPlaying then
    begin
        if ClipList.Count > 0 then
        begin
            Txt := ClipList[LB.ItemIndex];
            if LB.ItemIndex = LB.Count - 1 then
            begin
                LB.ItemIndex := 0;
            end else
            begin
                LB.ItemIndex := LB.ItemIndex + 1;
            end;
            LB.Repaint;
        end else
        begin
            Txt := DefaultItem;
        end;

        if WithCrLf then Txt := Txt + #13#10;

        Data := GlobalAlloc(GMEM_MOVEABLE, Length(Txt) + 1);
        Buff := GlobalLock(Data);
        StrCopy(Buff, PChar(Txt));
        SetClipboardData(CF_TEXT, Data);
        GlobalUnlock(Data);
    end;
end;

(*
	=======================================================
    RXgN^EfXgN^
	=======================================================
*)
procedure TMainForm.FormCreate(Sender: TObject);
begin
  inherited;
    // ^CgZbg
    Self.Caption := Application.Title;
    // `prbg}bv쐬
    Bitmap := TBitmap.Create;
    Bitmap.Canvas.Font.Assign(Self.Font);
    Bitmap.Width := Screen.Width;
    Bitmap.Height := LB.ItemHeight;
    // Nbv{[h̃eLXgۑXg쐬
    ClipList := TStringList.Create;
    // Rs[oNX
    ClipWatcher := TClipWatcher.Create(GetClip);
    // y[XgoNX
    PasteWatcher := TPasteWatcher.Create(Render);
    // [h
    ClipMode := cmNone;
    // ǂł悢
    DefaultItem := '';
    // ~[hɐݒ
    InitStopping();
    // {ۑpXݒ
    DataFilePath := ApplicationDirectory + DATA_FILE;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
    // ~[h
    ClipMode := cmNone;
    // Rs[oNX
    ClipWatcher.Free;
    // y[XgoNX
    PasteWatcher.Free;
    // ۑXg
    ClipList.Free;
  inherited;
end;

(*
	=======================================================
    [hύX
	=======================================================
*)

// L^[h
procedure   TMainForm.InitRecording();
begin
    ClipMode := cmRecording;
    pnlState.Caption := 'NOW RECORDING...';
    pnlState.Color := clRed;
end;

// Đ[h
procedure   TMainForm.InitPlaying();
begin
    ClipMode := cmPlaying;
    pnlState.Caption := 'NOW PLAYING...';
    pnlState.Color := clBlue;
    PasteWatcher.SetRender();
end;

// ~[h
procedure   TMainForm.InitStopping();
begin
    pnlState.Caption := 'NOW STOPPING...';
    pnlState.Color := clGray;
    ClipMode := cmNone;
end;

(*
	=======================================================
    j[Cxg & {^Cxg
	=======================================================
*)

procedure TMainForm.mnuRecordClick(Sender: TObject);
begin
    InitRecording();
end;

procedure TMainForm.mnuPlayClick(Sender: TObject);
begin
    InitPlaying();
end;

procedure TMainForm.mnuStopClick(Sender: TObject);
begin
    InitStopping();
end;

procedure TMainForm.mnuFirstClick(Sender: TObject);
begin
    if ClipList.Count > 0 then LB.ItemIndex := 0;
end;

procedure TMainForm.mnuLastClick(Sender: TObject);
begin
    if ClipList.Count > 0 then LB.ItemIndex := ClipList.Count - 1;
end;

procedure TMainForm.mnuExitClick(Sender: TObject);
begin
    Self.Close;
end;

procedure TMainForm.mnuConfClick(Sender: TObject);
begin
    ConfForm.ShowModal;
end;

procedure TMainForm.mnuInfoClick(Sender: TObject);
begin
    OpenInfoForm();
end;

procedure TMainForm.mnuClearClick(Sender: TObject);
begin
    ClipList.Clear;
    LB.Count := ClipList.Count;
end;

procedure TMainForm.mnuOpenClick(Sender: TObject);
begin
    if Open.Execute then
    begin
        ClipList.LoadFromFile(Open.FileName);
        LB.Count := ClipList.Count;
        if ClipList.Count > 0 then LB.ItemIndex := 0;
    end;
end;

procedure TMainForm.mnuSaveClick(Sender: TObject);
begin
    if Save.Execute then
    begin
        ClipList.SaveToFile(Save.FileName);
    end;
end;

procedure TMainForm.mnuCRLFClick(Sender: TObject);
begin
    WithCrLf := mnuCRLF.Checked;
end;

procedure TMainForm.mnuFLDRClick(Sender: TObject);
begin
    ExecuteFile(ApplicationDirectory);
end;

(*
	=======================================================
    ێeListBoxւ̕\
	=======================================================
*)
procedure TMainForm.LBData(Control: TWinControl; Index: Integer;
  var Data: String);
begin
    Data := ClipList[Index];
end;

procedure TMainForm.LBDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
    Text: string;
    ARect: TRect;
    BackCol, FontCol: TColor;
    X, Y: integer;
begin
    ARect := Bounds(0, 0, Rect.Right - Rect.Left, Rect.Bottom - Rect.Top);
    Text := ClipList[Index];
    Text := Format('%4.4d(%5dbytes):%s', [Index + 1, Length(Text), Text]);

    BackCol := RGB(240, 240, 240);
    FontCol := clWindowText;
    if odSelected in State then
    begin
        BackCol := RGB(204, 204, 255);
    end;

    Bitmap.Canvas.Brush.Color := BackCol;
    Bitmap.Canvas.Font.Color := FontCol;
    X := 5;
    Y := (ARect.Bottom - Bitmap.Canvas.TextHeight(Text)) div 2;

    Bitmap.Canvas.TextRect(ARect, X, Y, Text);

    BitBlt(LB.Canvas.Handle, Rect.Left, Rect.Top, ARect.Right, ARect.Bottom,
        Bitmap.Canvas.Handle, 0, 0, SRCCOPY);
end;

(*
	=======================================================
    ̑Cxg
	=======================================================
*)
procedure TMainForm.FormShow(Sender: TObject);
begin
    Open.FileName := DataFilePath;
    Save.FileName := DataFilePath;

    LoadForm(Self);
    mnuCRLF.Checked := AppIni.ReadBool('Application', mnuCRLF.Name, mnuCRLF.Checked);
    WithCrLf := mnuCRLF.Checked;

    if FileExists(DataFilePath) then
    begin
        ClipList.LoadFromFile(DataFilePath);
        LB.Count := ClipList.Count;
        LB.ItemIndex := 0;
    end;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    ClipList.SaveToFile(DataFilePath);
    SaveForm(Self);
    AppIni.WriteBool('Application', mnuCRLF.Name, mnuCRLF.Checked);
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
    if Key = VK_ESCAPE then Self.Close;
end;

procedure TMainForm.LBKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
    Index: integer;
begin
    if Key = VK_DELETE then
    begin
        Index := LB.ItemIndex;
        if Index <> -1 then
        begin
            ClipList.Delete(Index);
            LB.Count := ClipList.Count;
            if ClipList.Count > 0 then LB.ItemIndex := 0;
        end;
    end;
end;

end.
